MODULE compute_diagnostics_mod
  USE icosa
  IMPLICIT NONE
  SAVE
  PRIVATE

  ! fake array dimensions, for interfaces
  INTEGER, PARAMETER :: iim_jjm_i=1, iim_jjm_u=1, iim_jjm_v=1, llm_=1, llm1=1, nqdyn_=1

  INTERFACE

    SUBROUTINE comp_rhodz(flag,ps,rhodz)
      IMPORT
      LOGICAL, INTENT(IN) :: flag
      REAL(rstd),INTENT(IN)  :: ps(iim_jjm_i)
      REAL(rstd),INTENT(OUT) :: rhodz(iim_jjm_i,llm_)
    END SUBROUTINE comp_rhodz

    SUBROUTINE comp_pression(ps, p, offset)
      IMPORT
      INTEGER, INTENT(IN) :: offset
      REAL(rstd), INTENT(IN) :: ps(iim_jjm_i)
      REAL(rstd), INTENT(OUT) :: p(iim_jjm_i,llm_)
    END SUBROUTINE comp_pression

    SUBROUTINE comp_temperature(pmid,q,temp)
      IMPORT
      REAL(rstd),INTENT(IN)    :: pmid(iim_jjm_i, llm_)
      REAL(rstd),INTENT(IN)    :: q(iim_jjm_i, llm_, nqdyn_)
      REAL(rstd),INTENT(INOUT) :: temp(iim_jjm_i, llm_)
    END SUBROUTINE comp_temperature

    SUBROUTINE comp_hydro_press(rhodz, theta_rhodz, ps, p)
      IMPORT
      REAL(rstd),INTENT(IN)  :: rhodz(iim_jjm_i, llm_)
      REAL(rstd),INTENT(IN)  :: theta_rhodz(iim_jjm_i, llm_, nqdyn_)
      REAL(rstd),INTENT(OUT) :: ps(iim_jjm_i)
      REAL(rstd),INTENT(OUT) :: p(iim_jjm_i, llm_)
    END SUBROUTINE comp_hydro_press

    SUBROUTINE comp_vert_interp(pmid,in,out,pval)
      IMPORT
      REAL(rstd),INTENT(IN) :: pmid(iim_jjm_i, llm_)
      REAL(rstd),INTENT(IN) :: in(iim_jjm_i, llm_)
      REAL(rstd),INTENT(OUT):: out(iim_jjm_i)
      REAL(rstd),INTENT(IN) :: pval
    END SUBROUTINE comp_vert_interp

  END INTERFACE

  PROCEDURE(comp_rhodz),       POINTER :: compute_rhodz   => NULL()
  PROCEDURE(comp_pression),    POINTER :: compute_pression => NULL(), compute_pression_mid => NULL()
  PROCEDURE(comp_temperature), POINTER :: compute_temperature => NULL()
  PROCEDURE(comp_hydro_press), POINTER :: compute_hydrostatic_pressure => NULL()
  PROCEDURE(comp_vert_interp), POINTER :: compute_vertical_interp => NULL()

  PUBLIC :: compute_rhodz, compute_pression, compute_pression_mid, compute_temperature, compute_hydrostatic_pressure, &
       compute_vertical_interp

END MODULE compute_diagnostics_mod
